import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';
import { timer } from 'rxjs/observable/timer';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { GetDateTimePipe } from '../pipes/getdatetime.pipe';

import { ScheduleService } from './schedule.service';

import { Slot } from '../models/slot';


export class ScheduleDataSource implements DataSource<Slot> {
    private scheduleSubject = new BehaviorSubject<Slot[]>([]);
    private loadingSubject = new BehaviorSubject<boolean>(false);
    private countdownSubject = new BehaviorSubject<number>(undefined);

    public loading$ = this.loadingSubject.asObservable();
    public countdown$ = this.countdownSubject.asObservable();

    private lastCountdown = 0;

    constructor(private scheduleService: ScheduleService) { }

    loadSchedule(disabled: boolean) {
        this.loadingSubject.next(true);

        this.scheduleService.getSchedule(disabled)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe((schedule: Slot[]) => {
                schedule = this.sorted(schedule);

                this.scheduleSubject.next(schedule);

                timer(0, 1000).subscribe(() => {
                    if (schedule.length == 0) {
                        return;
                    }

                    let nearest = (new GetDateTimePipe()).transform(schedule[0]);

                    if (schedule.length > 1) {
                        let nearest2 = (new GetDateTimePipe()).transform(schedule[1]);

                        if (nearest.getTime() > nearest2.getTime()) {
                            schedule = this.sorted(schedule);
                            this.scheduleSubject.next(schedule);
                            return;
                        }
                    }

                    let countdown = nearest.getTime() - Date.now();
                    let h = countdown - this.lastCountdown;

                    if (h > 0 && h < 2000) {
                        countdown = this.lastCountdown;
                    }

                    this.lastCountdown = countdown;

                    this.countdownSubject.next(countdown);
                });
            });
    }

    private sorted(schedule) {
        schedule.sort(function(a, b) {
            a = (new GetDateTimePipe()).transform(a);
            b = (new GetDateTimePipe()).transform(b);
            return a.getTime() - b.getTime();
        });

        return schedule;
    }

    connect(collectionViewer: CollectionViewer): Observable<Slot[]> {
        console.log('Connecting data source');
        return this.scheduleSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.scheduleSubject.complete();
        this.loadingSubject.complete();
        this.countdownSubject.complete();
    }
}
